#include <stdio.h>
#include <stdlib.h>

#define TAG_NAME   0
#define TAG_GAMEID 1
#define TAG_DESC   2
#define TAG_CHEAT  3
#define TAG_MASTER 4
#define TAG_END    0xFF

/*
 * .qch format:
 
 <type> <len> [data]
 
 00 xx gamename (start of new game)
 02 xx cheat description
 03 08 normal cheat
 04 08 master cheat 
 */

void out_item(int type, int len, void *data)
{
	unsigned char c = type, d = len;
	write(1, &c, 1);
	write(1, &d, 1);
	write(1, data, len);
}

void add_file(char *filename)
{
	FILE *f = fopen(filename, "r");
	if (!f)
	{
		perror(filename);
		return;
	}
	
	int have_master = 1;
	int have_name = 0;
	int in_cheat = 0;
	
	int skipfirst = 0; // 1;
	int doskip = 0;
	
	while (1)
	{
		char line[256];
		if (!fgets(line, 256, f))
			break;
		line[strlen(line)-1] = 0;
		if (line[strlen(line)-1] == '\r')
			line[strlen(line)-1] = 0;
		if (!*line)
		{
			if (in_cheat)
				have_master = 0;
			in_cheat = 0;
			continue;
		}
		
		if (!have_name)
		{
			out_item(TAG_NAME, strlen(line) + 1, line);
			have_name = 1;
		} else if (!in_cheat)
		{
			if (!have_master)
			{
				out_item(TAG_DESC, strlen(line) + 1, line);
			}
			doskip = skipfirst;
			in_cheat = 1;
		} else
		{
			unsigned long c[2];
			if (sscanf(line, "%08x %08x", &c[0], &c[1]) != 2)
			{
				fprintf(stderr, "cheat error (file %s, line %s)\n", filename, line);
				return;
			}
			c[0] = htonl(c[0]);
			c[1] = htonl(c[1]);
			if (!doskip)
				out_item(have_master ? TAG_CHEAT : TAG_MASTER, 8, c);
			else
				--doskip;
		}
	}
	fclose(f);
}

int main(int argc, char **argv)
{
	unsigned char hdr[256];
	
	memset(hdr, 0, 256);
	memcpy(hdr, "QCHT", 4);
	
	write(1, hdr, 256);
	
	int i;
	for (i = 1; i < argc; ++i)
		add_file(argv[i]);
	
	out_item(TAG_END, 0, 0);
}
